;7-bit binary to 3x 7-segment converter
;ATTiny2313 @ 1 MHz (internal osc.)
;
;PA0   = Display 2 (tens) anode through PNP
;PA1   = Display 3 (units) anode through PNP
;PB0-6 = Binary input, bits 0-6
;PB7   = Display 1 (hundreds) anode through PNP
;PD0-6 = Display Segment cathodes a-g

.include "../include/tn2313def.inc"
.def	temp = R16
.def	temp2 = R17
.def	disp1 = R18
.def	disp2 = R19
.def	disp3 = R20
.def	data = R21

.cseg
.org 0x000
	rjmp	reset

zahlen:
	;     1    2    4    8   16   32   64
	.db 1,0, 2,0, 4,0, 8,0, 6,1, 2,3, 4,6
segmente:
	;     0     1     2     3     4     5     6     7     8     9
	.db 0x40, 0x79, 0x24, 0x30, 0x19, 0x12, 0x02, 0x78, 0x00, 0x10

reset:
	ldi	temp, RAMEND
	out	SPL, temp

	ldi	temp, 0x03
	out	DDRA, temp
	out	PORTA, temp
	ldi	temp, 0x80
	out	DDRB, temp
	out	PORTB, temp
	ldi	temp, 0x7F
	out	DDRD, temp
	out	PORTD, temp

loop:
	clr	disp1
	clr	disp2
	clr	disp3
	in	data, PINB
	andi	data, 0x7F
	ldi	ZL, LOW(zahlen*2)
	ldi	ZH, HIGH(zahlen*2)
	ldi	temp2, 0x01
convert:
	mov	temp, data
	and	temp, temp2
	breq	convert_noadd
	lpm	temp, Z+
	add	disp3, temp
	lpm	temp, Z+
	adc	disp2, temp
	cpi	disp3, 10
	brlo	overflow2
	subi	disp3, 10
	inc	disp2
overflow2:
	cpi	disp2, 10
	brlo	convert_next
	subi	disp2, 10
	inc	disp1
	rjmp	convert_next
convert_noadd:
	adiw	ZH:ZL, 2
convert_next:
	lsl	temp2
	cpi	temp2, 0x80
	brlo	convert
	mov	temp, disp1
	rcall	encode
	mov	disp1, temp
	mov	temp, disp2
	rcall	encode
	mov	disp2, temp
	mov	temp, disp3
	rcall	encode
	mov	disp3, temp

	out	PORTD, disp1
	cbi	PORTB, 7
	rcall	delay
	sbi	PORTB, 7
	out	PORTD, disp2
	cbi	PORTA, 0
	rcall	delay
	sbi	PORTA, 0
	out	PORTD, disp3
	cbi	PORTA, 1
	rcall	delay
	sbi	PORTA, 1
	
	rjmp	loop


encode:
	ldi	ZL, LOW(segmente*2)
	ldi	ZH, HIGH(segmente*2)
	add	ZL, temp
	ldi	temp, 0
	adc	ZH, temp
	lpm	temp, Z
	ret


delay:
	ldi	temp, 5
	clr	temp2
delay_loop:
	dec	temp2
	brne	delay_loop
	dec	temp
	brne	delay_loop
	ret